<?php

namespace Drupal\page_cache_single\StackMiddleware;

use Drupal\Core\Database\Database;
use Drupal\page_cache\StackMiddleware\PageCache;
use Symfony\Component\HttpFoundation\Request;

/**
 * Extends the PageCache middleware to handle 404 pages differently.
 *
 * This class customizes the caching behavior for 404 pages in Drupal.
 * It ensures that 404 pages are cached separately from other pages
 * by generating unique cache IDs for them. This is useful for improving
 * performance and minimizing size of the cache_page database table.
 */
class PageCacheSingle extends PageCache {

  /**
   * {@inheritdoc}
   */
  protected function getCacheId(Request $request) {
    if (isset($this->cid)) {
      return $this->cid;
    }

    $path = $request->getPathInfo();

    if ($this->isPath404($path)) {
      $cid_parts = [
        $request->getSchemeAndHttpHost(),
        '404',
        $request->getLocale(),
      ];
    }
    else {
      $cid_parts = [
        $request->getSchemeAndHttpHost() . $path,
        $request->getRequestFormat(NULL),
      ];
    }

    $this->cid = implode(':', $cid_parts);
    return $this->cid;
  }

  /**
   * Determines if the given path corresponds to a 404 page.
   *
   * @param string $path
   *   The path to check.
   *
   * @return bool
   *   TRUE if the path is a 404, FALSE otherwise.
   */
  protected function isPath404(string $path) {
    if ($path === '/') {
      return FALSE;
    }

    $sql = "SELECT pattern_outline FROM {router} WHERE :path LIKE CONCAT(pattern_outline, '%') AND pattern_outline != '/'";
    $result = Database::getConnection()->query($sql, [':path' => $path])->fetchField();
    if ($result) {
      return FALSE;
    }

    $path_noslash = rtrim($path, '/');
    $sql = "SELECT id FROM {path_alias} WHERE alias = :alias";
    $result = Database::getConnection()->query($sql, [':alias' => urldecode($path_noslash)])->fetchField();
    if ($result) {
      return FALSE;
    }

    return TRUE;
  }

}
